==> slashdb_diag-1.0.0-py3-none-any.whl: slashdb_diag/main.py <==
from datetime import datetime
from pathlib import Path
from zipfile import ZipFile
import socket
from functools import partial
import stat
from typing import Optional, List, Set
from tempfile import TemporaryDirectory
import shutil
import os

import click

from .config import (
    get_log_paths_from_config,
    get_nginx_log_paths_from_config,
    get_apikeys_from_config,
    get_config_paths_from_config,
    get_configs_from_config,
    get_nginx_log_format_from_config,
    ApiKeyConfig,
)
from .secrets import create_secret_mapping, replace_secrets
from .logs import LogFile
from .files import DiagFile

INI_PATH_FALLBACKS = [Path("/etc/slashdb/slashdb.ini")]
NGINX_CONF_PATH_FALLBACKS = [
    Path("/etc/slashdb/nginx.conf"),
    Path("/etc/nginx/nginx.conf"),
]


@click.command()
@click.option(
    "--ini",
    type=click.Path(exists=True, dir_okay=False, path_type=Path),
    help="Path to slashdb.ini.",
)
@click.option(
    "--nginx-conf",
    type=click.Path(exists=True, dir_okay=False, path_type=Path),
    help="Path to nginx.conf.",
)
@click.option(
    "--start",
    type=click.DateTime(formats=["%Y-%m-%dT%H:%M:%S"]),
    default=datetime.min,
    help="Start date of the logs, YYYY-MM-DDTHH:mm:ss format.",
)
@click.option(
    "--end",
    type=click.DateTime(formats=["%Y-%m-%dT%H:%M:%S"]),
    default=datetime.max,
    help="End date of the logs, YYYY-MM-DDTHH:mm:ss format.",
)
@click.option(
    "--configs/--no-configs",
    default=True,
    help="Whether to include config files.",
    show_default=True,
)
@click.option(
    "--zip/--no-zip",
    default=True,
    help="Whether to zip the resulting directory.",
    show_default=True,
)
@click.option(
    "--combined/--no-combined",
    default=True,
    help="Whether to combine log files to a single file.",
    show_default=True,
)
@click.argument(
    "log_file_paths_list",
    metavar="LOG_FILES",
    type=click.Path(exists=True, dir_okay=False, path_type=Path),
    nargs=-1,
)
def main(
    ini: Optional[Path],
    nginx_conf: Optional[Path],
    start: datetime,
    end: datetime,
    configs: bool,
    zip: bool,
    combined: bool,
    log_file_paths_list: List[Path],
):
    """
    Tool for collecting log files and config files to help with troubleshooting.

    \b
    Examples:
    $ slashdb_diag
    Will try to find configs and logs in standard locations
    and will save them in a zip file.

    \b
    $ slashdb_diag --ini /path/to/slashdb.ini --nginx-conf /path/to/nginx.conf
    Will get configs and logs based on the paths provided.

    \b
    $ slashdb_diag /var/log/slashdb/slashdb.log /var/log/slashdb/uwsgi.log --no-combined --no-zip
    Will get only these two log files and save them in a folder.

    \b
    $ slashdb_diag --start 2023-11-17T16:06:10 --end 2023-11-17T17:06:10
    Will get only logs in the given time frame.
    """
    ini_path_fallbacks = INI_PATH_FALLBACKS
    if venv := os.environ.get("VIRTUAL_ENV"):
        ini_path_fallbacks.append(Path(venv) / "etc")
    if not ini:
        for path in ini_path_fallbacks:
            if (
                path.exists()
                and path.is_file()
                and (path.stat().st_mode & stat.S_IRUSR)
            ):
                ini = path
                break
    if not nginx_conf:
        for path in NGINX_CONF_PATH_FALLBACKS:
            if (
                path.exists()
                and path.is_file()
                and (path.stat().st_mode & stat.S_IRUSR)
            ):
                nginx_conf = path
                break
    if not nginx_conf and not log_file_paths_list:
        click.echo(
            "WARNING: nginx.conf not found. " 
            "Logs from nginx will not be collected."
        )

    click.echo(f"slashdb.ini path: {str(ini)}")
    click.echo(f"nginx.conf path: {str(nginx_conf)}")

    nginx_format_str = None
    if nginx_conf:
        nginx_format_str = get_nginx_log_format_from_config(nginx_conf)

    log_files: List[LogFile] = []
    log_file_paths = set(log_file_paths_list)
    if log_file_paths_list:
        open_func = partial(LogFile.open, nginx_format_str=nginx_format_str)
        log_files = list(map(open_func, log_file_paths))
    elif ini:
        log_file_paths = get_log_paths_from_config(ini)
        if nginx_conf:
            log_file_paths |= get_nginx_log_paths_from_config(nginx_conf)
        open_func = partial(LogFile.open_like, nginx_format_str=nginx_format_str)
        log_files = list(map(open_func, log_file_paths))
    else:
        click.echo("slashdb.ini not found and no log files provided: aborting!")
        return

    log_files = list(filter(None, log_files))

    log_file_paths_display = "\n\t".join(map(str, log_file_paths))
    click.echo(f"Collecting log files:\n\t{log_file_paths_display}")

    for log_file in log_files:
        log_file.filter_by_datetime(start, end)

    if combined:
        log_files = [LogFile.combine(log_files)]

    config_paths: Set[Path] = set()
    config_files: List[DiagFile] = []
    api_key_config: Optional[ApiKeyConfig] = None
    if configs:
        if ini:
            api_key_config = get_apikeys_from_config(ini)
            config_paths |= get_config_paths_from_config(ini)
            config_paths.add(ini)
            config_files.extend(get_configs_from_config(ini))
        if nginx_conf:
            config_paths.add(nginx_conf)
            config_files.append(DiagFile.open(nginx_conf))

    config_paths_display = "\n\t".join(map(str, config_paths))
    click.echo(f"Collecting config files:\n\t{config_paths_display}")

    if not any(map(lambda lf: lf.lines, log_files)):
        if not config_files:
            click.echo(
                "Resulting log files would be empty "
                "and no config files collected: aborting!"
            )
            return
        click.echo("WARNING: Resulting log files are empty.")

    if not log_files and not config_files:
        click.echo("No log files or config files to collect: aborting!")
        return

    if ini:
        api_key_config = get_apikeys_from_config(ini)
        secret_mapping = create_secret_mapping(api_key_config)
        secret_replace_func = partial(replace_secrets, secret_mapping)
        for diag_file in log_files + config_files:
            diag_file.map_lines(secret_replace_func)
    else:
        click.echo(
            "WARNING: slashdb.ini not found. "
            "Passwords and API keys will not be scrambled."
        )

    if zip:
        filename = get_filename()
        unique_filename = get_unique_filename(f"{filename}.zip")
        zip_file = ZipFile(unique_filename, mode="w")
        click.echo(f"Writing to {unique_filename}")
        for file in log_files:
            zip_file.writestr(f"{filename}/logs/{file.name}", "".join(file.lines))
        for file in config_files:
            zip_file.writestr(f"{filename}/configs/{file.name}", "".join(file.lines))
    else:
        with TemporaryDirectory() as tmpdir:
            for file in log_files:
                log_folder = Path(tmpdir).joinpath("logs")
                log_folder.mkdir(exist_ok=True)
                with open(log_folder.joinpath(file.name), "w", encoding="utf-8") as f:
                    f.writelines(file.lines)
            for file in config_files:
                config_folder = Path(tmpdir).joinpath("configs")
                config_folder.mkdir(exist_ok=True)
                with open(
                    config_folder.joinpath(file.name), "w", encoding="utf-8"
                ) as f:
                    f.writelines(file.lines)
            dirname = get_unique_filename(get_filename())

            click.echo(f"Writing to {dirname}")
            shutil.copytree(tmpdir, dirname)
    click.echo("Done!")


def get_filename() -> str:
    hostname = socket.gethostname()
    run_datetime = datetime.now().strftime("%Y%m%d-%H%M")
    return f"diag-{hostname}-{run_datetime}"


def get_unique_filename(filename: str) -> str:
    path = Path(filename)
    n = 1
    result = path
    while result.exists():
        result = result.with_name(f"{path.stem}_{n}{path.suffix}")
        n += 1
    return result


if __name__ == "__main__":
    main()

==> slashdb_diag-1.1.1-py3-none-any.whl: slashdb_diag/main.py <==
from datetime import datetime
from pathlib import Path
from zipfile import ZipFile
import socket
from functools import partial
from typing import Optional, List, Set
from tempfile import TemporaryDirectory
import shutil

import click
from slashdb.cli.utils import (
    get_ini_from_fallbacks,
    get_nginx_conf_from_fallbacks,
)

from .config import (
    get_log_paths_from_config,
    get_nginx_log_paths_from_config,
    get_apikeys_from_config,
    get_config_paths_from_config,
    get_configs_from_config,
    get_nginx_log_format_from_config,
    get_log_formats_from_ini,
    generate_log_header,
    ApiKeyConfig,
)
from .secrets import create_secret_mapping, replace_secrets
from .logs import LogFile
from .files import DiagFile


@click.command()
@click.option(
    "--ini",
    type=click.Path(exists=True, dir_okay=False, path_type=Path),
    help="Path to slashdb.ini.",
)
@click.option(
    "--nginx-conf",
    type=click.Path(exists=True, dir_okay=False, path_type=Path),
    help="Path to nginx.conf.",
)
@click.option(
    "--start",
    type=click.DateTime(formats=["%Y-%m-%dT%H:%M:%S"]),
    default=datetime.min,
    help="Start date of the logs, YYYY-MM-DDTHH:mm:ss format.",
)
@click.option(
    "--end",
    type=click.DateTime(formats=["%Y-%m-%dT%H:%M:%S"]),
    default=datetime.max,
    help="End date of the logs, YYYY-MM-DDTHH:mm:ss format.",
)
@click.option(
    "--configs/--no-configs",
    default=True,
    help="Whether to include config files.",
    show_default=True,
)
@click.option(
    "--zip/--no-zip",
    default=True,
    help="Whether to zip the resulting directory.",
    show_default=True,
)
@click.option(
    "--combined/--no-combined",
    default=True,
    help="Whether to combine log files to a single file.",
    show_default=True,
)
@click.option(
    "--with-headers/--no-with-headers",
    default=True,
    help="Whether to add column headers to the log files.",
    show_default=True,
)
@click.argument(
    "log_file_paths_list",
    metavar="LOG_FILES",
    type=click.Path(exists=True, dir_okay=False, path_type=Path),
    nargs=-1,
)
def main(
    ini: Optional[Path],
    nginx_conf: Optional[Path],
    start: datetime,
    end: datetime,
    configs: bool,
    zip: bool,
    combined: bool,
    with_headers: bool,
    log_file_paths_list: List[Path],
):
    """
    Tool for collecting log files and config files to help with troubleshooting.

    \b
    Examples:
    $ slashdb-diag
    Will try to find configs and logs in standard locations
    and will save them in a zip file.

    \b
    $ slashdb-diag --ini /path/to/slashdb.ini --nginx-conf /path/to/nginx.conf
    Will get configs and logs based on the paths provided.

    \b
    $ slashdb-diag /var/log/slashdb/slashdb.log /var/log/slashdb/uwsgi.log --no-combined --no-zip
    Will get only these two log files and save them in a folder.

    \b
    $ slashdb-diag --start 2023-11-17T16:06:10 --end 2023-11-17T17:06:10
    Will get only logs in the given time frame.
    """
    ini = ini or get_ini_from_fallbacks()
    nginx_conf = nginx_conf or get_nginx_conf_from_fallbacks()
    if not nginx_conf and not log_file_paths_list:
        click.echo(
            "WARNING: nginx.conf not found. "
            "Logs from nginx will not be collected."
        )

    click.echo(f"slashdb.ini path: {str(ini)}")
    click.echo(f"nginx.conf path: {str(nginx_conf)}")

    nginx_format_str = None
    log_formats = set()
    if ini and with_headers:
        log_formats |= get_log_formats_from_ini(ini)
    if nginx_conf:
        if nginx_format_str := get_nginx_log_format_from_config(nginx_conf):
            log_formats.add(nginx_format_str)

    log_files: List[LogFile] = []
    log_file_paths = set(log_file_paths_list)
    if log_file_paths_list:
        open_func = partial(LogFile.open, nginx_format_str=nginx_format_str)
        log_files = list(map(open_func, log_file_paths))
    elif ini:
        log_file_paths = get_log_paths_from_config(ini)
        if nginx_conf:
            log_file_paths |= get_nginx_log_paths_from_config(nginx_conf)
        open_func = partial(LogFile.open_like, nginx_format_str=nginx_format_str)
        log_files = list(map(open_func, log_file_paths))
    else:
        click.echo("slashdb.ini not found and no log files provided: aborting!")
        return

    log_files = list(filter(None, log_files))

    log_file_paths_display = "\n\t".join(map(str, log_file_paths))
    click.echo(f"Collecting log files:\n\t{log_file_paths_display}")

    for log_file in log_files:
        log_file.filter_by_datetime(start, end)

    if combined:
        log_files = [LogFile.combine(log_files)]

    config_paths: Set[Path] = set()
    config_files: List[DiagFile] = []
    api_key_config: Optional[ApiKeyConfig] = None
    if configs:
        if ini:
            api_key_config = get_apikeys_from_config(ini)
            config_paths |= get_config_paths_from_config(ini)
            config_paths.add(ini)
            config_files.extend(get_configs_from_config(ini))
        if nginx_conf:
            config_paths.add(nginx_conf)
            config_files.append(DiagFile.open(nginx_conf))

    config_paths_display = "\n\t".join(map(str, config_paths))
    click.echo(f"Collecting config files:\n\t{config_paths_display}")

    if not any(map(lambda lf: lf.lines, log_files)):
        if not config_files:
            click.echo(
                "Resulting log files would be empty "
                "and no config files collected: aborting!"
            )
            return
        click.echo("WARNING: Resulting log files are empty.")

    if not log_files and not config_files:
        click.echo("No log files or config files to collect: aborting!")
        return

    if ini:
        api_key_config = get_apikeys_from_config(ini)
        secret_mapping = create_secret_mapping(api_key_config)
        secret_replace_func = partial(replace_secrets, secret_mapping)
        for diag_file in log_files + config_files:
            diag_file.map_lines(secret_replace_func)
    else:
        click.echo(
            "WARNING: slashdb.ini not found. "
            "Passwords and API keys will not be scrambled."
        )

    if with_headers:
        log_header = generate_log_header(log_formats)
        for log in log_files:
            log.prepend_line(log_header)

    if zip:
        filename = get_filename()
        unique_filename = get_unique_filename(f"{filename}.zip")
        zip_file = ZipFile(unique_filename, mode="w")
        click.echo(f"Writing to {unique_filename}")
        for file in log_files:
            zip_file.writestr(f"{filename}/logs/{file.name}", "".join(file.lines))
        for file in config_files:
            zip_file.writestr(f"{filename}/configs/{file.name}", "".join(file.lines))
    else:
        with TemporaryDirectory() as tmpdir:
            for file in log_files:
                log_folder = Path(tmpdir).joinpath("logs")
                log_folder.mkdir(exist_ok=True)
                with open(log_folder.joinpath(file.name), "w", encoding="utf-8") as f:
                    f.writelines(file.lines)
            for file in config_files:
                config_folder = Path(tmpdir).joinpath("configs")
                config_folder.mkdir(exist_ok=True)
                with open(
                    config_folder.joinpath(file.name), "w", encoding="utf-8"
                ) as f:
                    f.writelines(file.lines)
            dirname = get_unique_filename(get_filename())

            click.echo(f"Writing to {dirname}")
            shutil.copytree(tmpdir, dirname)
    click.echo("Done!")


def get_filename() -> str:
    hostname = socket.gethostname()
    run_datetime = datetime.now().strftime("%Y%m%d-%H%M")
    return f"diag-{hostname}-{run_datetime}"


def get_unique_filename(filename: str) -> str:
    path = Path(filename)
    n = 1
    result = path
    while result.exists():
        result = result.with_name(f"{path.stem}_{n}{path.suffix}")
        n += 1
    return result


if __name__ == "__main__":
    main()
